#ifndef DB_H
#define DB_H

#include "common.h"

QString      getHsTable(const QString &snapShot);
QString      getHfTable(const QString &fork);
QString      getPsTable(const QString &snapShot);
QString      getPfTable(const QString &fork);
QString      getPTable(const QString &datasetName, const QSqlDatabase &dbObj);
QString      getHTable(const QString &datasetName, const QSqlDatabase &dbObj);
QString      getDataset(const QString &path);
QStringList  getAllHTables();
QStringList  getAllPTables();
QStringList  getAllFSTables(const char *sSig, const char *fSig);
QSqlDatabase getDbObj(int &retCode);
quint32      getIno(const QString &path);
void         getAllDatasetNames(const char *pSig, const char *hSig, QHash<QString, uint> &list, uint modes);
void         getAllSnapShots(QHash<QString, uint> &list);
void         getAllForks(QHash<QString, uint> &list);
int          getMode(const QString &path, const QString &dataset, const QSqlDatabase &dbObj);
int          getMode(QString path);
int          countRows(QSqlQuery &query);
bool         delDataset(const QString &name, int &retCode);
bool         cpyDataset(const QString &src, const QString &dst, int &retCode);
bool         updateIndex(quint32 ino, const QString &dataset, int &retCode);
bool         flushHashCache(quint32 ino, int &retCode);
bool         flushHashCache(const Block &cache, int &retCode);
bool         procHashCache(Block &cache, const char *buff, quint64 &dataLen, bool &overFlow, int &retCode);
bool         dbExec(QSqlQuery &query, int &retCode, bool init = true);
bool         dbExec(QSqlQuery &query, const QString &qstr, int &retCode, bool init = false);
bool         checkPath(QString &path, const QSqlDatabase &dbObj, QString &dataset, int pathop, int &retCode);
bool         verifySnapshot(const QString &snapShot, int &retCode);
bool         createDataset(const QString &name, const QSqlDatabase &dbObj, DataSetTypes type, int &retCode);
bool         dataSetExists(const QString &name, const QSqlDatabase &dbObj);
bool         tableExists(const QString &table, const QSqlDatabase &dbObj);
bool         addBlk(const QByteArray &hash, const QByteArray &blk, int &retCode);
bool         isDir(int mode);
bool         isSym(int mode);
bool         isSnapshot(const QString &name);
bool         isFork(const QString &name);
bool         createFile(const QString &path, mode_t mode, const QByteArray &data, int &retCode);
bool         createRoot(const QString &pTable, QSqlQuery &query, int &retCode);
bool         createDir(const QString &path, mode_t mode, int &retCode);
bool         removeItem(const QString &path, int &retCode);
bool         rePath(const QString &oldPath, const QString &newPath, int &retCode);
bool         deepCpy(const QString &src, const QString &dst, const QSqlDatabase &dbObj, const QString &srcDataset, const QString &dstDataset, int &retCode);
bool         truncateBlks(const QString &path, quint64 oldSize, quint64 newSize, int &retCode);
bool         allocate(const QString &path, quint64 currentSize, quint64 offs, quint64 len, int &retCode);
bool         getBlkData(const QString &path, char *buff, quint64 dataOffs, quint64 dataLen, int &retCode);
bool         setBlkData(const QString &path, const char *buff, quint64 dataOffs, quint64 dataLen, int &retCode);
bool         setHash(quint32 ino, bool exists, int blkNum, const QByteArray &newHash, const QString &dataset, const QSqlDatabase &dbObj, int &retCode);
bool         getDirQuery(QString path, QSqlQuery &query, int &retCode);
bool         getIndexQuery(const QString &path, QSqlQuery &query, int &retCode);
bool         checkVersion(int &retCode);
bool         dbSetup(int &retCode);

class Cleaner : public QObject
{
    Q_OBJECT

private:

    bool           firstRun;
    QList<quint32> inoQueue;
    QList<quint64> mtQueue;
    QList<QString> dsQueue;
    QTimer        *timer;

private slots:

    void exeGarbageCollect();

public:

    void garbageCollect();

    explicit Cleaner(QObject *parent = nullptr);
};

#endif // DB_H
